// -----------------------------------------------------------------------
// MatchPoint - DKFZ translational registration framework
//
// Copyright (c) German Cancer Research Center (DKFZ),
// Software development for Integrated Diagnostics and Therapy (SIDT).
// ALL RIGHTS RESERVED.
// See mapCopyright.txt or
// http://www.dkfz.de/en/sidt/projects/MatchPoint/copyright.html
//
// This software is distributed WITHOUT ANY WARRANTY; without even
// the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the above copyright notices for more information.
//
//------------------------------------------------------------------------
/*!
// @file
// @version $Revision: 1316 $ (last changed revision)
// @date    $Date: 2016-04-13 14:50:09 +0200 (Mi, 13 Apr 2016) $ (last change date)
// @author  $Author: floca $ (last changed by)
// Subversion HeadURL: $HeadURL: http://svn.dkfz-heidelberg.de/sbr/Sources/SBR-Projects/MatchPoint/trunk/Code/Core/test/mapTestFieldGenerationFunctor.h $
*/

#ifndef __MAP_TEST_FIELD_GENERATION_FUNCTOR_H
#define __MAP_TEST_FIELD_GENERATION_FUNCTOR_H

#include "mapTransformGenerationFunctor.h"
#include "mapArtifactGenerator.h"

#include "itkImageRegionIterator.h"

namespace map
{
	namespace testing
	{
		template<unsigned int VInputDimensions, unsigned int VOutputDimensions>
		class TestFieldGenerationFunctor : public
			map::core::functors::TransformGenerationFunctor<VInputDimensions, VOutputDimensions>
		{
		public:
			/*! Standard class typedefs. */
			typedef TestFieldGenerationFunctor<VInputDimensions, VOutputDimensions>  Self;
      typedef map::core::functors::TransformGenerationFunctor<VInputDimensions, VOutputDimensions> Superclass;
			typedef itk::SmartPointer<Self>        Pointer;
			typedef itk::SmartPointer<const Self>  ConstPointer;
			typedef typename Superclass::InFieldRepresentationType InFieldRepresentationType;
      typedef typename Superclass::TransformType TransformType;
      typedef typename Superclass::TransformPointer TransformPointer;

			itkTypeMacro(TestFieldGenerationFunctor, FieldGenerationFunctor);

			static Pointer New(const InFieldRepresentationType* pInFieldRepresentation)
			{
				Pointer smartPtr = new Self(pInFieldRepresentation);
				smartPtr->UnRegister();
				return smartPtr;
			}

      virtual TransformPointer generateTransform() const
			{
          typename ::itk::GenericVectorFieldTransform< ::map::core::continuous::ScalarType, VInputDimensions, VOutputDimensions>::Pointer transform = wrapFieldInTransform<VOutputDimensions>(testing::generateSumField<VOutputDimensions>(Superclass::_spInFieldRepresentation));
          transform->SetUseNullPoint(this->_useNullPoint);

          _spCurrentTransform = transform;
          return _spCurrentTransform;
			}

      mutable TransformPointer _spCurrentTransform;

		protected:

			TestFieldGenerationFunctor(const InFieldRepresentationType* pInFieldRepresentation):
				Superclass(pInFieldRepresentation)
			{};

			virtual ~TestFieldGenerationFunctor() {};

		private:
			TestFieldGenerationFunctor(const Self&);  //purposely not implemented
			void operator=(const Self&);  //purposely not implemented
		};


	} // end namespace testing
} // end namespace map

#endif
